#!/usr/local/bin/perl -w
#!/usr/bin/perl -w
# --------------------------------------------------------------------
$Version = 'database_split-1.0.3';
#
#  Copyright (c) 2003-2005 Lymeware Corporation,  All Rights Reserved
#  THIS IS PROPRIETARY SOURCE CODE OF LYMEWARE CORPORATION
# 
# $Header: /projRCS/cabsviewer/src/database_split.pl,v 1.5 2005/12/18 04:39:22 aclark Exp $
#
# This software has been developed as part of the CABS Viewer Product 
# from the Lymeware Corporation <www.lymeware.com> 
#
# See the README file for licensing, distribution, usage, and 
# installation information.  See the Changes file for known problems 
# and version info.
#
sub usage2 {
    die <<"EndUsage";
usage: database_split [-hedvx] [-b <binary path>] CABS_BOS_file [CABS_BOS_file [...]]
$Version
Process a CABS BOS input file and generates a split database output format im multiple files, specifically for database import use.
Display Options:
     -h  Help -- just display this message and quit.
     -d  Do display all debug messages to STDERR.
     -e  Display all invalid log entries on STDERR.
     -v  Verbose display (to STDERR) of each entry processed.
     -x  Display all requests of nonexistant files to STDERR.
Input Options:
     -b <binary path> Force the use of <binary path> for CABS Viewer location.
    ...  Process the sequence of CABS BOS files.
EndUsage
}

#
# ---------------------------------------------------------------------------
# Set the default configuration options:
#cygwin#$BinaryPath        = "./cv_csp.exe"; # Path to CABS Viewer binary
#win#$BinaryPath        = "cv_csp.exe"; # Path to CABS Viewer binary
$BinaryPath        = "cabsviewer.exe"; # Path to CABS Viewer binary
###$BinaryPath        = "./cv_db.exe"; # Path to CABS Viewer binary

# ==========================================================================
# The following are unlikely to need changing:

$DebugPrint        = 0;    # Display   valid log entries on STDERR
$Verbose           = 0;    # Display   valid log entries on STDERR
$PrintInvalids     = 0;    # Display invalid log entries on STDERR
$PrintNonexist     = 0;    # Display nonexistant file requests on STDERR

# ==========================================================================
# Get the command-line options

require "getopts.pl";
# autoflush output
$| = 1;
my $opt_d = $opt_e = $opt_h = $opt_v = $opt_x = $opt_b = "";
&Getopts('hedvxb:');
if ($@ || $opt_h) { &usage2; }

if ($opt_d) { $DebugPrint    = 1; }
if ($opt_e) { $PrintInvalids = 1; }
if ($opt_v) { $Verbose       = 1; }
if ($opt_x) { $PrintNonexist = 1; }

if ($opt_b) { $BinaryPath    = $opt_b; }

# ==========================================================================
# Now process each BOSfile left on command-line or just the default one
# 

$Updated   = time;                     # Get the current date-time stamp

print(STDERR "$Version: $Updated\n") if $Verbose;

$bos_files=0;
if ($ARGV[0])
{
    while ($input_file = shift)
    {
        &process_bos($input_file);
        &process_split($input_file);
	$bos_files++;
    }
}
else
{
    &process_bos($input_file);
    &process_split($input_file);
    $bos_files++;
}

print(STDERR "Done processing BOS file(s)\n") if $Verbose;

if ($bos_files == 0) {die "There were no BOS files found\n";}

exit(0);

# ==========================================================================
# ==========================================================================
# Read the passed-in csv file and generate CSR-specific report
#
sub process_bos
{
  local($thisfile) = @_;

  print(STDERR "Processing BOS file \"$thisfile\"\n") if $Verbose;

  if (not -f $thisfile) {
	print STDERR "ERROR: " . $thisfile . " cannot be found or opened.\n";
	goto err;
  }

  # run the CV binary with the correct args
  #print(STDERR " system($BinaryPath . \" --csv --csrrecords --output=\" . $thisfile . \".csv \" . $thisfile . \" >> genCSRreport.log 2> /dev/null\");\n");
  my $sys_string = $BinaryPath . " --csv --database --output=" . $thisfile . ".csv " . $thisfile . " >> database_split.log";

  print(STDERR " system(" . $sys_string . ");\n") if $DebugPrint;
  system($sys_string);

  if (not -f $thisfile . ".csv") {
	print STDERR "ERROR: " . $thisfile . ".csv was NOT created.\n";
	goto err;
  }
err:
  return;
}


# ==========================================================================
# Read the passed-in csv file and generate multiple single record files in 
# database-split format (a single header and CSV data records)
#
sub process_split
{
  local($thisfile) = @_;

  print(STDERR "Processing CSV file \"$thisfile\".csv\n") if $Verbose;

  if (!open (LOG, "<$thisfile.csv"))
  {
      print(STDERR "Error opening CSV file: $thisfile.csv\n");
      return;
  }

  # set all values to empty
  my $linein = 0;
  my $lineout = 0;

  LINE: while (<LOG>)
  {

    $linein++;
    print STDERR "line: $linein\r" if $DebugPrint;
    $dataline = $_;

    #
    # First, parse the database record format into its three basic components
    #

    $rname = "";
    $rindex = "";
    $value = "";
    # this next line will force record number into rname, index into rindex,
    # and everything else into value (including all the rest of the 
    # comma-delimited data)
    ($rname, $rindex, $value) = split(/,/,$dataline,3);

    if (!$rname) {
       	print(STDERR "$linein missing rname - skipping [$dataline]\n");
        next LINE;
	}
    # NOTE: rindex can == 0 for the first index record
	#if (!$rindex) {
       	#print(STDERR "$linein missing rindex - skipping [$dataline]\n");
	#next LINE;
	#}
    if (!$value) {
       	print(STDERR "$linein missing value - skipping [$dataline]\n");
        next LINE;
	}

    if ($rindex eq " Index") {
	# then header (with index/rindex) not needed so skip if found
    	if ( -f "$thisfile.$rname.csv") {
       		print(STDERR "skipping header for $rname\n") if $DebugPrint;
        	next LINE;
    	}
    }

    # open a new record-specific file
    #open(OUT,">$thisfile.$rname.csv") or die;
    # open existing file
    open(OUT,">>$thisfile.$rname.csv") or die;

    if ($value) {
        $value =~ s/\\n//;            # Remove any newlines
        $value =~ s/\x0a//;           # Remove any LFs
        $value =~ s/\x0d//;           # Remove any CRs
        $value =~ s/\s+$//g;          # Remove any trailing spaces
    }

    #if (!$rindex)
    #{
    #print(STDERR "$.:$dataline") if $PrintInvalids;
    #print 
    #next LINE;
    #}

    #
    # Now we can write lines in to the record-specific CSV file
    #

    print(STDERR "$rname $rindex $value\n") if $Verbose;
    #print("### $rname, $rindex, $value\n") if $DebugPrint;
    print(OUT "$rname, $rindex, $value\n");

    $lineout++;
    close OUT;
  }
  close LOG;
  print STDERR "generated $lineout lines\n" if $DebugPrint;
}

# ==========================================================================
exit;
1;

# EOF database_split.pl
