#!/usr/local/bin/perl -w
#!/usr/bin/perl -w
# --------------------------------------------------------------------
$Version = 'genCSRreport-1.0.1';
#
#  Copyright (c) 2002-2005 Lymeware Corporation,  All Rights Reserved
#  THIS IS PROPRIETARY SOURCE CODE OF LYMEWARE CORPORATION
# 
# $Header: /projRCS/cabsviewer/src/genCSRreport.pl,v 1.1 2005/11/04 14:11:22 aclark Exp $
#
# This software has been developed as part of the CABS Viewer Product 
# from the Lymeware Corporation <www.lymeware.com> 
#
# See the README file for licensing, distribution, usage, and 
# installation information.  See the Changes file for known problems 
# and version info.
#
sub usage2 {
    die <<"EndUsage";
usage: genCSRreport [-hedvx] [-b <binary path>] CABS_BOS_file [CABS_BOS_file [...]]
$Version
Process a CABS BOS input file and output a CSR Report summary in CSV format, specifically for Excel use.
Display Options:
     -h  Help -- just display this message and quit.
     -d  Do display all debug messages to STDOUT.
     -e  Display all invalid log entries on STDERR.
     -v  Verbose display (to STDERR) of each entry processed.
     -x  Display all requests of nonexistant files to STDERR.
Input Options:
     -b <binary path> Force the use of <binary path> for CABS Viewer location.
    ...  Process the sequence of CABS BOS files.
EndUsage
}

#
# ---------------------------------------------------------------------------
# Set the default configuration options:
#cygwin#$BinaryPath        = "./cv_csp.exe"; # Path to CABS Viewer binary
#win#$BinaryPath        = "cv_csp.exe"; # Path to CABS Viewer binary
$BinaryPath        = "cabsviewer.exe"; # Path to CABS Viewer binary

# ==========================================================================
# The following are unlikely to need changing:

$DebugPrint        = 0;    # Display   valid log entries on STDERR?
$Verbose           = 0;    # Display   valid log entries on STDERR?
$PrintInvalids     = 0;    # Display invalid log entries on STDERR?
$PrintNonexist     = 0;    # Display nonexistant file requests on STDERR?

# ==========================================================================
# Get the command-line options

require "getopts.pl";
my $opt_d = $opt_e = $opt_h = $opt_v = $opt_x = $opt_b = "";
&Getopts('hedvxb:');
if ($@ || $opt_h) { &usage2; }

if ($opt_d) { $DebugPrint        = 1; }
if ($opt_e) { $PrintInvalids     = 1; }
if ($opt_v) { $Verbose           = 1; }
if ($opt_x) { $PrintNonexist     = 1; }

if ($opt_b) { $BinaryPath    = $opt_b; }

# ==========================================================================
# Now process each BOSfile left on command-line or just the default one
# 

$Updated   = time;                     # Get the current date-time stamp

if ($Verbose) { print(STDERR "$Version: $Updated\n"); }

$bos_files=0;
if ($ARGV[0])
{
    while ($input_file = shift)
    {
        &process_bos($input_file);
        &process_csv($input_file . ".csv");
	$bos_files++;
    }
}
else
{
    &process_bos($input_file);
    &process_csv($input_file . ".csv");
    $bos_files++;
}

if ($Verbose)
{ print(STDERR "Done processing BOS files, now doing summary\n"); }

if ($bos_files == 0) {die "There was no matching data to summarize.\n";}

exit(0);

# ==========================================================================
# ==========================================================================
# Read the passed-in csv file and generate CSR-specific report
#
sub process_bos
{
  local($thisfile) = @_;

  if ($Verbose) { print(STDERR "Processing BOS file \"$thisfile\"\n"); }

  if (not -f $thisfile) {
	print "ERROR: " . $thisfile . " cannot be found or opened.\n";
	goto err;
  }

  if ($DebugPrint) {
	  #print(STDERR " system($BinaryPath . \" --csv --csrrecords --output=\" . $thisfile . \".csv \" . $thisfile . \" >> genCSRreport.log 2> /dev/null\");\n");
	  #print(STDERR " system(" . $BinaryPath . " --csv --csrrecords --output=" . $thisfile . ".csv " . $thisfile . " >> genCSRreport.log 2> /dev/null\n");
	  print(STDERR " system(" . $BinaryPath . " --csv --csrrecords --output=" . $thisfile . ".csv " . $thisfile . "\n");
  }
  #system($BinaryPath . " --csv --csrrecords --output=" . $thisfile . ".csv " . $thisfile . " >> genCSRreport.log 2> /dev/null");
  	system($BinaryPath . " --csv --csrrecords --output=" . $thisfile . ".csv " . $thisfile . " >> genCSRreport.log");

  if (not -f $thisfile . ".csv") {
	print "ERROR: " . $thisfile . ".csv was NOT created.\n";
	goto err;
  }
err:
  return;
}


# ==========================================================================
# Read the passed-in csv file and generate CSR-specific report
#
sub process_csv
{
  local($thisfile) = @_;

  if ($Verbose) { print(STDERR "Processing CSV file \"$thisfile\"\n"); }

  if (!open (LOG, "<$thisfile"))
  {
      print(STDERR "Error opening CSV file: $thisfile\n");
      return;
  }

  # open the new output file for CSR report
  open(OUT,">$thisfile"."_report.csv") or die;

  # set all values to empty
  my $linein = 0;
  my $lineout = 0;
  $csv_btn = $csv_usoc = $csv_fid = $csv_fid_data = "";
  $csv_usoc_quantity = $csv_local_unit_rate = $csv_local_billed_amount = "";
  $csv_serv_feat_subtotal = $csv_local_subtotal = "";

  # write headers
  print OUT "Record Number, 40 15 05, 40 15 10, 40 15 10, 40 15 20, 40 15 20, 40 15 20, 40 15 25, 40 15 25\n";
  print OUT "Field within Record, FID Data, USOC, FID Data, UCOS Quantity, Local Unit Rate, Local Billed Amount, Services and Features Subtotal, Local Subtotal\n";
  print OUT "=\"Description\",BTN, USOC,=\"Line Number\",=\"USOC Quantity\",=\"Unit Rate\",=\"Total Rate\",=\"USOC Sub Total\",=\"BTN Total\"\n";


  LINE: while (<LOG>)
  {
    $linein++;
    if ($DebugPrint) { print STDERR "line: $linein\r"; }
    $saveline = $_;
    $dataline = $_;

    #
    # First, parse the new format into its seven basic components
    #

    ($rname, $fname, $value) = split(',',$dataline,3);

    if ($fname) {
    	$fname =~ s/\?.*$//;          # Remove any trailing query information
    }
    if ($value) {
        $value =~ s/\\n//;            # Remove any newlines
        $value =~ s/\x0a//;           # Remove any LFs
        $value =~ s/\x0d//;           # Remove any CRs
        $value =~ s/\s+$//g;          # Remove any trailing spaces
    }

    if (!$fname)
    {
        print(STDERR "$.:$saveline") if $PrintInvalids;
        next LINE;
    }

    #
    # Now we can generate lines in the CSR report CSV file
    #

    if ($Verbose)
    {
        print(STDERR "$rname $fname $value\n");
    }
    if (!$rname)
    {
       if ($Verbose) { print("$fname: [$value]\n"); }
       #print(OUT "$fname: $value\n");
    }
    else
    {
       if ($Verbose) { print("$rname: $fname: [$value]\n"); }
       #print(OUT "$rname: $fname: $value\n");
    }

    if ($fname eq "FID")
    {
	if ($DebugPrint) { print "found FID\n"; }
	$csv_fid = $value;
    } elsif ($fname eq "USOC")
    {
	if ($DebugPrint) { print "found USOC\n"; }
    	$csv_usoc = $value;
    } elsif ($fname eq "FID Data")
    {
	if ($DebugPrint) { print "found FID_DATA\n"; }
	# if FID = TN then csv_btn = value else csv_fid_data = value
	if ($csv_fid eq "TN")
	{
		$csv_btn = "=\"$value\"";
		$csv_fid = ""; # reset for next record
	}
	else
	{
		$csv_fid_data = $value;
                if ($Verbose) {
		    print "### , $csv_btn, $csv_usoc, $csv_fid_data,";
		    print " $csv_usoc_quantity, $csv_local_unit_rate, $csv_local_billed_amount, ,\n";
		}
		print OUT " ,$csv_btn, $csv_usoc, $csv_fid_data,";
		print OUT " $csv_usoc_quantity, $csv_local_unit_rate, $csv_local_billed_amount, ,\n";
    		$lineout++;
		$csv_btn = $csv_usoc = $csv_fid = $csv_fid_data = "";
	}
    } elsif ($fname eq "USOC Quantity")
    {
	if ($DebugPrint) { print "found USOC Quantity\n"; }
    	$csv_usoc_quantity = $value;
    } elsif ($fname eq "Local Unit Rate")
    {
	if ($DebugPrint) { print "found Local Unit Rate\n"; }
    	$csv_local_unit_rate = $value;
    } elsif ($fname eq "Local Billed Amount")
    {
	if ($DebugPrint) { print "found Local_Billed_Amount\n"; }
    	$csv_local_billed_amount = $value;
        if ($Verbose) {
	   print "### , $csv_btn, $csv_usoc, $csv_fid_data,";
	   print " $csv_usoc_quantity, $csv_local_unit_rate, $csv_local_billed_amount, ,\n";
	}
	print OUT " ,$csv_btn, $csv_usoc, $csv_fid_data,";
	print OUT " $csv_usoc_quantity, $csv_local_unit_rate, $csv_local_billed_amount, ,\n";
    	$lineout++;
	$csv_usoc_quantity = $csv_local_unit_rate = $csv_local_billed_amount = "";
    } elsif ($fname eq "Services And Features Subtotal")
    {
	if ($DebugPrint) { print "found Service And Features Subtotal\n"; }
    	$csv_serv_feat_subtotal = $value;
    } elsif ($fname eq "Local Subtotal")
    {
	if ($DebugPrint) { print "found Local_Subtotal\n"; }
    	$csv_local_subtotal = $value;
        if ($Verbose) {
	   print "### , , , , , , , $csv_serv_feat_subtotal, $csv_local_subtotal\n";
	}
	print OUT " , , , , , , , $csv_serv_feat_subtotal, $csv_local_subtotal\n";
    	$lineout++;
	$csv_serv_feat_subtotal = $csv_local_subtotal = "";
    }
  }
  close LOG;
  close OUT;
  print STDERR "generated $lineout lines\n";
}

# ==========================================================================
exit;
1;

# EOF genCSRreport.pl
