#!/usr/local/bin/perl -w
#!/usr/bin/perl -w
# --------------------------------------------------------------------
$Version = 'processBatch-1.0.1';
#
#  Copyright (c) 2002-2005 Lymeware Corporation,  All Rights Reserved
#  THIS IS PROPRIETARY SOURCE CODE OF LYMEWARE CORPORATION
# 
# $Header: /projRCS/cabsviewer/src/processBatch.pl,v 1.1 2005/09/22 12:34:56 aclark Exp $
#
# This software has been developed as part of the CABS Viewer Product 
# from the Lymeware Corporation <www.lymeware.com> 
#
# See the README file for licensing, distribution, usage, and 
# installation information.  See the Changes file for known problems 
# and version info.
#
sub usage2 {
    die <<"EndUsage";
usage: processBatch [-hdv] [-b <binary path>] [-e <extension>] 
 [-t <output format>] [-a <command line arguments>] <CABS_BOS_directory>
$Version
Process a single directory containing CABS BOS input files (with default extension of .bos) and run CABS Viewer (with args) on each.
Display Options:
     -h  Help -- just display this message and quit.
     -d  Do display all debug messages to STDERR.
     -v  Verbose display (to STDERR) of each entry processed.
Input Options:
     -b <binary path> Force the use of <binary path> for CABS Viewer location.
     -e <extension> Treat any files with this extension to an imput file
     	(defualt is .bos).
     -t <output format> can be one of the following:
     	text, csv, xml, html or database
     -a <command line arguments> to pass to CABS Viewer for each file processed.
     <CABS_BOS_directory> full path to directory of BOS files.
EndUsage
}

#
# ---------------------------------------------------------------------------
# Set the default configuration options:
#cygwin#$BinaryPath        = "./cv_csp.exe"; # Path to CABS Viewer binary
#win#$BinaryPath        = "cv_csp.exe"; # Path to CABS Viewer binary
$BinaryPath        = "cabsviewer.exe"; # Path to CABS Viewer binary
###$BinaryPath        = "./cv_db.exe"; # Path to CABS Viewer binary
$Extension	   = ".bos";  # look for files like *.bos
$OutputExtension   = "";  # force output format to this
$CvArgs		   = ""; # No additional args by default

# ==========================================================================
# The following are unlikely to need changing:

$DebugPrint        = 0;    # Display   valid log entries on STDERR
$Verbose           = 0;    # Display   valid log entries on STDERR

# ==========================================================================
# Get the command-line options

require "getopts.pl";
# autoflush output
$| = 1;
my $opt_d = $opt_e = $opt_h = $opt_v = $opt_a = $opt_b = $opt_t = "";
&Getopts('he:a:dvb:t:');
if ($@ || $opt_h) { &usage2; }

if ($opt_d) { $DebugPrint    = 1; }
if ($opt_v) { $Verbose       = 1; }

if ($opt_b) { $BinaryPath    = $opt_b; }
if ($opt_e) { $Extension     = $opt_e; }
if ($opt_a) { $CvArgs	     = $opt_a; }
if ($opt_t) {
	if ($opt_t eq "text") {
		$CvArgs = $CvArgs . " --text";
		$OutputExtension     = ".text";
	}
	elsif ($opt_t eq "csv") {
		$CvArgs = $CvArgs . " --csv";
		$OutputExtension     = ".csv";
	}
	elsif ($opt_t eq "xml") {
		$CvArgs = $CvArgs . " --xml";
		$OutputExtension     = ".xml";
	}
	elsif ($opt_t eq "html") {
		$CvArgs = $CvArgs . " --html";
		$OutputExtension     = ".html";
	}
	elsif ($opt_t eq "database") {
		$CvArgs = $CvArgs . " --database --csv";
		$OutputExtension     = ".db";
	}
	else {
		die "Invalid OutputType of $opt_t encountered.  Only text, csv, xml, html and database are supported.\n";
	}
}
else {
	# need to set DEFAULT output type.
	$CvArgs = $CvArgs . " --csv";
	$OutputExtension     = ".csv";
}

# ==========================================================================
# Now process each BOS directory left on command-line or just the default one
# 

$Updated   = time;                     # Get the current date-time stamp

print(STDERR "$Version: $Updated\n") if $Verbose;

$bos_files=0;
$bos_dirs=0;
while ($input_dir = shift)
{
	print "input_dir = $input_dir\n" if $DebugPrint;
	opendir(DIR, $input_dir) || die "Cannot open directory [". $input_dir . "]: $!";
	while ($file = readdir(DIR)) {
		print "file = $file\n" if $DebugPrint;
		$full_path = $input_dir . "/" . $file;
		if (-f $full_path) {
			if ($file =~ /$Extension$/i) {
				print STDERR "## Found file $file\n" if $Verbose;
    				&process_bos($full_path);
				$bos_files++;
			}
		}
	}
	$bos_dirs++;
}
closedir(DIR);

print(STDERR "Done processing BOS file(s)\n") if $Verbose;

if ($bos_files == 0) {die "There were no BOS files found\n";}
if ($bos_dirs == 0) {die "There were no BOS directories found\n";}

exit(0);

# ==========================================================================
# ==========================================================================
# Read the passed-in BOS file and generate output with CvArgs
#
sub process_bos
{
  local($thisfile) = @_;

  print(STDERR "Processing BOS file \"$thisfile\"\n") if $Verbose;

  if (not -f $thisfile) {
	print STDERR "ERROR: " . $thisfile . " cannot be found or opened.\n";
	goto err;
  }

  # run the CV binary with the correct args
  #print(STDERR " system($BinaryPath . \" --csv --csrrecords --output=\" . $thisfile . \".csv \" . $thisfile . \" >> genCSRreport.log 2> /dev/null\");\n");
  my $sys_string = $BinaryPath . $CvArgs. " --output=" . $thisfile . $OutputExtension . " " . $thisfile . " >> processBatch.log";

  print(STDERR " system(" . $sys_string . ");\n") if $DebugPrint;
  system($sys_string);

  if (not -f $thisfile . $OutputExtension) {
	print STDERR "ERROR: " . $thisfile . $OutputExtension . " was NOT created.\n";
	goto err;
  }
err:
  return;
}

# ==========================================================================
exit;
1;

# EOF processBatch.pl
